#ifndef EEPROM_DATA_H
#define EEPROM_DATA_H


// 0 -  =0, 1 -  =1, 2 -   =1   =0; 3 -   =0   =1
enum EMotorPowerBitType
{
	MPB_OFF = 0,
	MPB_ON = 1,
	MPB_RunningONsleepingOFF = 2,
	MPB_RunningOFFsleepingON = 3
};

enum ETermometersType
{
	DS1621 = 0,
	DS1820 = 1
};


#ifdef F_MICROSTEP
	#define F_SUBSTEPS 16
#else // F_MICROSTEP
	#define F_SUBSTEPS 2
#endif

#define  maxPossibleFilters 10

typedef struct 
{
	uint8_t			    m_empty;							// 0
	uint8_t				m_CountFilters;						// 1
	//---------------------------------------				// 
	uint16_t            m_MotorSpeed;						// 2
	uint8_t				m_MotorSleep;						// 4      
	uint8_t				m_MotorStep;						// 5 1 - , 2 -
	uint8_t				m_HomePresentInSensorFilter;		// 6 0 -      Home, 1 - 
	uint8_t				m_MotorPowerBitType;				// 7  EMotorPowerBitType

	//---------------------------------------
	uint16_t            m_FMotorSpeed;						// 8
	uint8_t				m_FMotorSleep;						// A        
	uint8_t				m_FMotorStep;						// B   - , 2 -
	uint16_t 			m_FBacklash;						// C
															
	uint8_t				m_TermometersType;					// E
	uint8_t				m_TermometersCount;					// F
	int16_t				m_FTK;								// 10
	uint16_t 			m_TemperatureAverage;				// 18
	uint16_t 			m_AutoTempMinReverseSteps;			// 20

	//---------------------------------------
	int16_t				m_PosCorr[maxPossibleFilters];		// 22
	int16_t				m_FocusCorr[maxPossibleFilters];	// 42

	char				m_Names[maxPossibleFilters][8];		// 62
	
	uint16_t			m_maxStepsChangeFilter;

} EEFilterWheelData;

#if defined (__AVR_ATmega8__) || defined (__AVR_ATmega16__) || defined (__AVR_ATmega32__)

#define EEPROM_SECTION  __attribute__ ((section (".eeprom")))

EEFilterWheelData	ee_FW_initial	EEPROM_SECTION = 
{
		0,	// empty
		9,	// m_CountFilters

		200,// m_MotorSpeed
		1,	// m_MotorSleep
		1,	// m_MotorStep
		0,	// m_HomePresentInSensorFilter
		3,	// m_MotorPowerBitType


		200,// m_FMotorSpeed
		1,	// m_FMotorSleep
		1,	// m_FMotorStep
		30,	// m_FBacklash

		0,	// m_TermometersType
		4,	// m_TermometersCount
		10,	// m_FTK
		100,// m_TemperatureAverage
		10, // m_AutoTempMinReverseSteps

		{
			9, -2, -6, 11, 4, -19, -6, 11, 8, 0
		}, 		// word m_PosCorr[maxPossibleFilters]

 		{
			0,0,0,0,0,0,0,0,0,0
		},		// word m_FocusCorr[maxPossibleFilters]

		{
			"Black",
			"Red",
			"Green",
			"Blue",
			"Light",
			"H-beta",
			"O-III",
			"H-alpha",
			"S-II",
			"IR-pass"
		},		// char m_Names[maxPossibleFilters][8]
		600     // uint16_t			m_maxStepsChangeFilter
};

uint8_t	ee_curAutoTermometer EEPROM_SECTION = -1;  // 0-first termometer  -1=none


#ifdef F_MICROSTEP
	// sqrt(sin(i/16*pi/2))*255
	int8_t	ee_microsteps[F_SUBSTEPS+1] EEPROM_SECTION = {0, 80, 113, 137, 158, 175, 190, 203, 214, 224, 233, 239, 245, 249, 253, 254, 255};
#endif


#if defined(_FOCUSER_) && defined(_FILTERWHEEL_)
	#if (LCD_WIDTH > 16)
		char ee_t0[]      EEPROM_SECTION ="Focuser FilterWheel";
	#else
		char ee_t0[]      EEPROM_SECTION ="* Focuser + FW *";
	#endif
#else
	#ifdef _FOCUSER_
		char ee_t0[]   EEPROM_SECTION ="*   Focuser    *";
	#else
		char ee_t0[]   EEPROM_SECTION ="* Filter Wheel *";
	#endif
#endif

char ee_t1[]		EEPROM_SECTION ="Alexander, Ivan";
char ee_t2[]		EEPROM_SECTION ="     Victor";

//char ee_SearchHomePos[]	EEPROM_SECTION ="Go Home";
//char ee_Filter[]			EEPROM_SECTION ="Filter";
//char ee_Search[]			EEPROM_SECTION ="Search";
//char ee_Found[]			EEPROM_SECTION ="Found";

char ee_keySpeedCaption[]	EEPROM_SECTION ="WheelSpd FocusSpd";
char ee_keyCorrectionCap[]	EEPROM_SECTION ="FilterCor FocusCor";
char ee_keyFocuserCap[]		EEPROM_SECTION ="Backlash K-Temp";
char ee_keyCommandCap[]		EEPROM_SECTION =" Go  Reset K-T  Avr";

char ee_filterChangeErr[]	EEPROM_SECTION ="Filter Change Error!!!";

#ifdef HAVE_SHUTDOWN
	char ee_Shutdown[]		EEPROM_SECTION ="Low Power";
	int16_t	ee_focuser_step_counter	EEPROM_SECTION	= 0;
	int16_t	ee_focuser_subshift	    EEPROM_SECTION	= 0;
	int16_t	ee_focuserFilterShift	EEPROM_SECTION	= 0;
	int8_t	ee_currFDirection       EEPROM_SECTION  = 1;
	int8_t	ee_currFBacklash        EEPROM_SECTION	= 0;
#endif

#endif // ATmega devices

#endif // EEPROM_DATA_H

