//Make: Name=filterwheel2, MCU=atmega16, Level=2, Debug=VMLab     =2
//: BODEN=BODLEVEL=SUT0=CKSEL3=CKSEL2=CKSEL1="0" 1=3
#include <avr/io.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
//#include <stdint.h>

#include <util/delay.h>
#include <math.h>

#define UART_UBRR_CALC(BAUD_,FREQ_) ((FREQ_)/((BAUD_)*16L)-1)
#define UART_BAUD_RATE 9600

#define CPUMHZ (F_CPU / 1000000UL)

//#define VOLTAGE_METER

#include "config_ports.h"
				  
void delay_loop_mks(int32_t delayMks)
{
	while(delayMks > 10240L)
	{
		_delay_loop_2((10240 / 4) * CPUMHZ);  //  4   
		delayMks -= 10240L;
	}
	_delay_loop_2((delayMks / 4) * CPUMHZ);
}

void delay_0dot1sec(uint8_t n)	// 0.1  * n 
{
	uint16_t i = (n * CPUMHZ) / 2;
	for(;i>0;i--)
	{
		_delay_loop_2(50000); // 50  F_CPU=4MHz
	}
}

#include "eeprom_io.h"
#include "eeprom_data.h"
#include "lcd_out.h"
		  
EEFilterWheelData	EEData;

#ifdef _FOCUSER_

#define MAXSENSORS 8

	#ifdef ONE_WIRE
	    #include "ds18x20.h"
	#endif

	#ifdef TWO_WIRE
		#include "i2c_ds1621_termometer.h"
	#endif

	unsigned char termometer_error;

#endif
	 
#ifdef VOLTAGE_METER
	#include "adc.h"
#endif // VOLTAGE_METER	 
	  			  
// key press variables
unsigned char keyFilterPrev = 0;
unsigned char keyFilterNext = 0;
unsigned char keyFocuserPrev = 0;
unsigned char keyFocuserNext = 0;

unsigned char keyCommand = 0;
unsigned char keySpeed = 0;
unsigned char keyFocuserCorr = 0;
unsigned char keyCorrection = 0;


#ifdef _FOCUSER_

	void showfocus(void);

	int16_t currFocusTermShift;

	enum eTemperState 
	{
		TSNotInited = 0,
		TSStarted = 1,
		TSReady = 2
	};
			 
#define Tscale	256L

	int32_t referenceTemperature;
	char	referenceTemperatureReady;
	int8_t  curTauto;

	//   
	int16_t			curT[MAXSENSORS];

	//   
	int32_t			avrT[MAXSENSORS];
	uint16_t		avrTCounts[MAXSENSORS];

	char			temperStates[MAXSENSORS];

	int16_t		  focuser_step_counter;
	int16_t		  focuser_step_subshift;


#ifdef HAVE_SHUTDOWN
	char canShutdown;
#endif

#endif

#ifdef _FILTERWHEEL_
	int8_t		  filterposition = -1;
	int16_t		  filters_step_counter = 0;
#endif

unsigned char automode;
unsigned char updateScreen;

	   	   
#include "step_motors.h"

//---------------------------------------------------------------------------
			  
char readkey(unsigned char port, unsigned char pin)
{
	portKeyLCD = ~_BV(port);
	_delay_loop_1(20);
	if( bit_is_clear(iportKeyRead, pin) )
		return 1;
	return 0;
}
	   
void readkeys(void)
{
	keyFocuserCorr  = readkey(keyPinFocuserCorr, keyInMenus);
	keyCommand		= readkey(keyPinCommand,     keyInMenus);
	keyCorrection	= readkey(keyPinCorrection,  keyInMenus);
	keySpeed		= readkey(keyPinSpeed,       keyInMenus);

	keyFocuserPrev  = readkey(keyPinFocuserPrev, keyInActions);
	keyFocuserNext	= readkey(keyPinFocuserNext, keyInActions);
	keyFilterPrev	= readkey(keyPinFilterPrev,  keyInActions);
	keyFilterNext	= readkey(keyPinFilterNext,  keyInActions);

	portKeyLCD = 0xff; //_BV(keyb_out_0) | _BV(keyb_out_1) | _BV(keyb_out_2) | _BV(keyb_out_3);
}
//----------------------------------------------------

void usart_init (void)
{
	UBRRH = (uint8_t)(UART_UBRR_CALC(UART_BAUD_RATE,F_CPU)>>8);
	UBRRL = (uint8_t)UART_UBRR_CALC(UART_BAUD_RATE,F_CPU);
	UCSRB |= ( 1 << TXEN ) |(1<<RXEN);
	UCSRB |= ( 1 << RXCIE); //   
	UCSRC |= ( 1 << URSEL )|( 3<<UCSZ0 );
}

void usart_transmit(unsigned char data)
{
	while (!(UCSRA & (1<<UDRE)));
	_delay_loop_2(1200);
	UDR=data;
} 
	  
void usart_transmit_string (char t2[])
{
	unsigned int b;
	b=0;
	while(t2[b]!=0)
	{
		usart_transmit(t2[b]);
		b++;
	}
}
 	  	 	  
//---------------------------------------------------------------------------
void writeSettingsToEEPROM(void)
{
	EEPROM_WRITE_STRING((char*)&EEData, sizeof(EEData), 0);

#ifdef F_MICROSTEP
	EEPROM_WRITE_STRING((char*)microStepPhase, F_SUBSTEPS+1, (uint16_t)&ee_microsteps);
#endif
}

#ifdef _FOCUSER_
	void setT_Kcompensation(char setTK);
#else
	void setT_Kcompensation(char setTK) {};
#endif // _FOCUSER_

	  
void readSettingsFromEEPROM(void)
{
	EEPROM_READ_STRING((char*)&EEData, sizeof(EEData), 0);
	
	if( EEData.m_CountFilters > maxPossibleFilters )
		EEData.m_CountFilters = maxPossibleFilters;
	
 	setT_Kcompensation( EEPROM_READ((uint16_t)&ee_curAutoTermometer) );
	
	EEData.m_MotorStep  = EEData.m_MotorStep  > 1 ? 2 : 1;


#ifdef F_MICROSTEP
	EEPROM_READ_STRING((char*)microStepPhase, F_SUBSTEPS+1, (uint16_t)&ee_microsteps);
	for( int8_t k = 1; k < F_SUBSTEPS; ++k )
	{
		microStepPhase[F_SUBSTEPS+k] = microStepPhase[F_SUBSTEPS-k];
	}
#endif


	// update wake-sleep status and power bit
#ifdef _FILTERWHEEL_
	wakeupMotor();
	sleepMotor();
#endif

#ifdef _FOCUSER_
	if( EEData.m_FMotorStep > F_SUBSTEPS )
		EEData.m_FMotorStep = F_SUBSTEPS;
	if( EEData.m_FMotorStep < 1 )
		EEData.m_FMotorStep = 1;

	wakeupFMotor();
	sleepFMotor();

	for( uint8_t k = 0; k < MAXSENSORS; ++k )
	{
		avrTCounts[k] = 0;
		temperStates[k] = TSNotInited;
		avrT[k] = 0;
	}
#endif
}

//----------------------------------------------------------------------------
int16_t lastFocuserFilterShift;

void sendFocuserCorrection(int16_t newFocuserShift)
{
	if( newFocuserShift > 10000 || newFocuserShift == lastFocuserFilterShift )
		return;

	if( lastFocuserFilterShift > 10000 )
	{
		lastFocuserFilterShift = newFocuserShift;
		return;
	}

	int16_t focuserShift = newFocuserShift - lastFocuserFilterShift;
	lastFocuserFilterShift = newFocuserShift;
	
#ifdef _FOCUSER_
	goFocusRelative(focuserShift);
//	showfocus();
#else
	if( focuserShift != 0 )
	{
		usart_transmit_string("FWmoveFocus");
		usart_transmit(focuserShift & 0xff);
		usart_transmit((focuserShift / 256) & 0xff);
	}
#endif // _FOCUSER_
}

#ifdef _FOCUSER_

void showfocus(void)
{
#ifndef _FILTERWHEEL_
	lcd_xy(0,0);
	lcd_write_string("Focuser");
#endif 
	lcd_xy(0,1);
	lcd_dat(currFDirection > 0 ? 0xD9 : 0xDA); //    
	lcd_write_int(focuser_step_counter, 5);
	lcd_dat(' ');
}

void setT_Kcompensation(char setTK)
{
	if( setTK == curTauto )
		return;

	referenceTemperature = 0;
	referenceTemperatureReady = 0;
	
	curTauto = setTK;
	if( curTauto >= EEData.m_TermometersCount )
		curTauto = EEData.m_TermometersCount - 1;
	if( curTauto < 0 )
		curTauto = -1;

	EEPROM_write((uint16_t)&ee_curAutoTermometer, curTauto);
}
		 
void processT_K(int32_t temperature)
{
// 	if( curTauto < 0 || temperature == 0x8000 ) 
// 		return;

	if( !referenceTemperatureReady )
	{
		referenceTemperatureReady = 1;
		referenceTemperature = temperature;
		currFocusTermShift = 0;
		return;
	}

	int32_t dif = temperature - referenceTemperature;

	dif	*= EEData.m_FTK;
	if( dif < 0 )
		dif	= -( (-dif + 50L * Tscale) / (100L * Tscale) );
	else
		dif	= (dif + 50L * Tscale) / (100L * Tscale);

	int16_t focusShift = dif;

//	int focusShift = (int)(((float)EEData.m_FTK * dif) / 100.f);
	if( focusShift != currFocusTermShift )
	{
		int16_t dif = focusShift - currFocusTermShift;
		int8_t dir = (dif > 0) ? 1 : -1;

		//        ,
		//         
		if( dir == currFDirection || (uint16_t)(dif*dir) >= EEData.m_AutoTempMinReverseSteps )
		{
			goFocusRelative(focusShift - currFocusTermShift);
			currFocusTermShift = focusShift;
		}
		updateScreen = 1;
	}
}

char cleanTemperature(uint8_t i)
{
	char res = avrTCounts[i] > 0;
	avrTCounts[i] = 0;
	temperStates[i] = TSNotInited;
	avrT[i] = 0;
	return res;
}

void doAverageTemper(uint8_t i)
{
	uint16_t count = avrTCounts[i];
	avrTCounts[i]++;
	if( avrTCounts[i] > 30000 )
		avrTCounts[i] = 10000;
	
	if( count > EEData.m_TemperatureAverage )
		count = EEData.m_TemperatureAverage;
	
	int32_t newT = Tscale * (int32_t)curT[i];
	int32_t oldT = avrT[i];
	avrT[i] = (oldT * count + newT) / (count + 1);
	temperStates[i] = TSReady;
}
			 
char readTemperature(uint8_t i)
{
#ifdef	TWO_WIRE
	if( checkI2Port() )
		return cleanTemperature(i);

#ifdef CLOUD_SENSOR
//	delay_0dot1sec(1);
	if( i == termN_cloud || i == termN_sensor )
	{
		uint16_t T = (i == termN_cloud) ? MLX_read_Tobj1_2() : MLX_read_Ta_2();
		if( !termometer_error )
		{
			curT[i] = MLX_calctemp(T);
// 			avrT[i] = Tscale * (int32_t)curT[i];
// 			avrTCounts[i]++;
//			temperStates[i] = TSReady;
			doAverageTemper(i);
 			return 1;
		}
//		return 0;
	}
// 
#endif

#endif
	
	switch(temperStates[i])
	{
	case TSNotInited:
		{
#ifdef	TWO_WIRE
			set_temperature_mode_ds1631(i, 0); 
			if( !termometer_error )
				temperature_start(i);
#endif
#ifdef	ONE_WIRE
			startDS18X20temperature(i);
#endif
			if( !termometer_error )
				temperStates[i] = TSStarted;
			return 0;
		}
	case TSStarted:	 
		{	 	
#ifdef	TWO_WIRE
			char fl = get_temperature_ready_flag(i);
			if( termometer_error )
				return cleanTemperature(i);
				
			if( (fl & 0x80) == 0 )
				return 0;
				
			_delay_loop_1(100);
			curT[i] = temperature_read(i);
#endif
#ifdef	ONE_WIRE
			if( !readDS18X20temperature(i, &curT[i]) )
				return 0;
#endif

			if( termometer_error || curT[i] == (int16_t)0x8000 )
				return cleanTemperature(i);

			doAverageTemper(i);
			return 1;
		}
	default:
//	case TSReady:
		{
#ifdef	TWO_WIRE
			temperature_start(i);
#endif
#ifdef	ONE_WIRE
			startDS18X20temperature(i);
#endif

			if( termometer_error )
				return cleanTemperature(i);
			
			temperStates[i] = TSStarted;
			return 0;
		}
	}
	return 0;
}

void showTemperature(int16_t temperature, char isAuto, uint8_t counter)
{
	if( temperature >= 0 ) 
	{
		if(temperature >= 10000)
		{
			lcd_write_uint(temperature / 100, 3);
		}
		else
		{
			lcd_dat('+'); 
			lcd_write_uint(temperature / 100, 2);
		}
		if( counter & 0x1 )
			lcd_dat(isAuto ? ';' : ',');
		else
			lcd_dat(isAuto ? ':' : '.');
		lcd_write_uint(temperature % 100, 2);
	}
	else
	{
		lcd_dat('-');lcd_write_uint((-temperature) / 100, 2);
		if( counter & 0x1 )
			lcd_dat(isAuto ? ';' : ',');
		else
			lcd_dat(isAuto ? ':' : '.');
		lcd_write_uint((-temperature) % 100, 2);
	}
}

void showAvrTemperature(uint8_t i, uint8_t processTK )
{
	if (!avrTCounts[i])
	{
		if( termometer_error != 255 && (termometer_error & (_BV(SDA) | _BV(SCL))) )
		{
			if( i == 0 && termometer_error & _BV(SDA) )
			{
				lcd_write_string("Stop");
				lcd_dat(0xD9);
			}
			if( i == 1 && termometer_error & _BV(SCL) )
			{
				lcd_write_string("Stop");
				lcd_dat(0xDA);
			}
			return;
		}

		if( temperStates[i] == TSNotInited )
		{
			lcd_dat(' ');
			lcd_dat('-');
			lcd_dat('T');
			lcd_dat('0' + i);
			lcd_dat('-');
			lcd_dat(' ');

			//			lcd_write_string(" -T");
			//			lcd_dat('0' + i);
			//			lcd_write_string("- ");
		}

		if( temperStates[i] == TSStarted )
			lcd_write_string ("-wait-");
		return;
	}

	showTemperature(avrT[i]/ Tscale, curTauto == i && !processTK, avrTCounts[i]);
}

#endif // _FOCUSER_

#ifdef _FILTERWHEEL_
				   
void showfilter(char* prefix, int8_t filterpos)
{
	lcd_xy(0,0);
	if( prefix )
		lcd_write_string( prefix );

	char* name = (filterpos < maxPossibleFilters && filterpos >= 0) ? EEData.m_Names[filterpos] : "???";

	//      
	unsigned char n = 0;
	while( (n < 8) && (name[n] != 0) )
		n++;

	n = (8 - n) / 2;

	unsigned char nn = n;
	for(; nn > 0; nn--) lcd_dat(' ');

	lcd_write_string_n( name, 8 - n );


#ifndef _FOCUSER_
	for(nn = 8; nn > 0; nn--) lcd_dat(' ');
#endif // _FOCUSER_
}	  

//=========================================================================
void filterPositioningError(void)
{
	sleepMotor();
	lcd_clean_fast();
	lcd_xy(0,0);
	lcd_write_eepromStr((uint16_t)&ee_filterChangeErr);
	delay_0dot1sec(10);
	lcd_clean_fast();
	filterposition = -1;
	updateScreen = 1;
}  

#define enshureSteps 3

int16_t goNextSensorFilterOn(int8_t dir, int16_t speed)
{
	uint16_t steps = 0;
	int8_t c = 0;
	while(1) //     
	{
		if( bit_is_set(iportSensors,SENSORFILTER) )
		{
			c = 0;
		}
		else
		{
			c++;
			if( c >= enshureSteps )
				break;
		}

		if(!speed) 
		{
			doStepMotor(dir);
		}
		else
		{
			doStepMotor2(dir, speed);
		}

		steps++;
		if( steps > EEData.m_maxStepsChangeFilter )
		{
			filterPositioningError();
			return -1;
		}
	}

//	if( steps > 100 )
// 	{
// 		lcd_dat(dir > 0 ? 0xD9 : 0xDA); //    
// 		lcd_dat('n');
// 		lcd_write_uint(steps, 3);
// 		lcd_dat(' ');
// 	}
//	delay_0dot1sec(4);

	return steps;
}


int16_t goNextSensorFilterOff(int8_t dir, int16_t speed)
{
	uint16_t steps = 0;
	int8_t c = 0;
	while(1) //     ->
	{
		if( bit_is_clear(iportSensors, SENSORFILTER) )
		{
			c = 0;
		}
		else
		{
			c++;
			if( c >= enshureSteps )
				break;
		}

		steps++;
		if(!speed) 
		{
			doStepMotor(dir);
		}
		else
		{
			doStepMotor2(dir, speed);
		}

		if( steps > EEData.m_maxStepsChangeFilter )
		{
			filterPositioningError();
			return -1;
		}
	}

// 	{
// 		lcd_dat(dir > 0 ? 0xD9 : 0xDA); //    
// 		lcd_dat('f');
// 		lcd_write_uint(steps, 3);
// 		lcd_dat(' ');
// 	}
	return steps;
}

#define goNextSensorFilterOffRet(dir) if( goNextSensorFilterOff(dir, 0) < 0 ) { return; }

#define goNextSensorFilterOnRet(dir) if( goNextSensorFilterOn(dir, 0) < 0 ) { return; }

void filterpositioning (void)
{   
	if( filterposition < 0 )
		return;
	
	uint16_t slowSpeed4 = EEData.m_MotorSpeed / 4 + 1;
	uint16_t slowSpeed2 = EEData.m_MotorSpeed / 2 + 1;
	
	wakeupMotor();
	lcd_xy(0,1);
	goNextSensorFilterOff(1, slowSpeed2);
	
	// 	//    
	// 	while (bit_is_clear(iportSensors,SENSORFILTER)) //     ->
	// 	{
	// 		doStepMotor2(1, slowSpeed2);
	// 	} 
	// 	doStepMotor2(1, slowSpeed2);
	// 	doStepMotor2(1, slowSpeed2);
	
	int16_t delta = goNextSensorFilterOn(-1, slowSpeed2);
	if( delta < 0 )
		return;
	
	// 	while( bit_is_set(iportSensors,SENSORFILTER) ) //     
	// 	{
	// 		doStepMotor2(-1, slowSpeed2);
	// 		delta++;
	// 	}
	
	int16_t stepcounter = 	goNextSensorFilterOff(-1, slowSpeed2);
	if( stepcounter < 0 )
		return;
	
	// 	while( bit_is_clear(iportSensors,SENSORFILTER) ) //     <-
	// 	{
	// 		doStepMotor2(-1, slowSpeed2);
	// 		stepcounter++;
	// 	} 

	{
		lcd_xy(0,1); 
		lcd_dat('l');
		lcd_write_uint(delta, 3);
		lcd_dat(' ');
		lcd_dat('w');
		lcd_write_uint(stepcounter, 3);
		lcd_dat(' ');
		lcd_dat(' ');
	}

	
	int16_t corr = EEData.m_PosCorr[filterposition];
	while( corr < 0 )
	{
		doStepMotor2(-1, slowSpeed2);
		corr++;
	}
	stepcounter = stepcounter / 2 + delta + corr;
	
	for (; stepcounter > 0; stepcounter--) //     + 
	{
		doStepMotor2(1, slowSpeed4 );
	}
	
	sleepMotor();

	delay_0dot1sec(10);
}


void findHomePosition(int8_t dir) 
{ 
	showfilter("Go", 0);
	
	wakeupMotor();
	
	
	uint16_t steps = 0;
	uint8_t sensFold = 0;
	uint8_t sensCount = 0;
	while( bit_is_set(iportSensors, SENSORHOME) || bit_is_set(iportSensors, SENSORFILTER) ) //    home 
	{
		uint8_t sensF = bit_is_set(iportSensors, SENSORFILTER);
		if( sensFold != sensF )
		{
			lcd_xy(sensF*8, 1); 
			lcd_write_uint(sensCount, 2);
			lcd_dat(' ');
			lcd_write_uint(steps, 3);
			lcd_dat(' ');
			lcd_dat(' ');

			steps = 0;
			sensCount++;
			sensFold = sensF;
		}
		doStepMotor(dir);
		steps++;
		if( steps > EEData.m_maxStepsChangeFilter || sensCount > 30 )
		{
			filterPositioningError();
			return;
		}
	}
	
	// 	if( steps > 100 )
	// 	{
	// 		lcd_xy(0,0); 
	// 		lcd_write_uint(steps, 3);
	// 		lcd_dat(' ');
	// 		lcd_dat(' ');
	// 		lcd_dat(' ');
	// 	}
	
	sendFocuserCorrection(EEData.m_FocusCorr[0]);
	filterposition = 0;
	filterpositioning();
	filters_step_counter = filters_step_counter & 0x07;
	
	showfilter(0, filterposition);
	sleepMotor();
	updateScreen = 1;
}


void gotoFilter(uint8_t newPos)
{
	char keySensitive = newPos & 0x80;
	newPos &= 0x7f;

	//      -    Home
	if( filterposition < 0 )
	{
		findHomePosition(1);
		if( filterposition < 0 )
			return;
	}

	int8_t diff = (newPos + EEData.m_CountFilters - filterposition) % EEData.m_CountFilters;
	
	if( diff > 0 ) 
		diff = (diff <= (EEData.m_CountFilters / 2)) ? 1 : -1;
	else
		diff = (EEData.m_PosCorr[filterposition] < 0) ? 1 : -1;


	//   0-    Home
	if( !newPos )
	{
		findHomePosition(diff);
		return;
	}

	updateScreen = 1;
	showfilter("Go", newPos);
	
	wakeupMotor();

	//      ,   
	if( EEData.m_PosCorr[filterposition] * diff < 0 || newPos == filterposition )
	{
		goNextSensorFilterOnRet(diff);
	}

	lcd_xy(0,0);

	int8_t dirH = diff > 0 ? 1 : 0;

	while( filterposition != newPos )
	{
		if( EEData.m_HomePresentInSensorFilter && (dirH + filterposition) == 1 )  //   Home  
		{
			goNextSensorFilterOffRet(diff);
			goNextSensorFilterOnRet(diff);
		}
		
		int16_t coff = goNextSensorFilterOff(diff, 0);
		if( coff < 0 )
			return;
		int16_t con = goNextSensorFilterOn(diff, 0);
		if( con < 0 )
			return;

		{
			lcd_xy(0,0); 
			lcd_dat('f');
			lcd_write_uint(coff, 3);
			lcd_dat(' ');
			lcd_dat('n');
			lcd_write_uint(con, 3);
			lcd_dat(' ');
			lcd_dat(' ');
		}
		
		filterposition = (filterposition + EEData.m_CountFilters + diff) % EEData.m_CountFilters;

		if( (filterposition == newPos) && (keySensitive != 0) )
		{
			wakeupMotor();
			readkeys();

			if( ((diff > 0) && keyFilterNext) || ((diff < 0) && keyFilterPrev) )
			{
				newPos = (newPos + EEData.m_CountFilters + diff) % EEData.m_CountFilters;
				showfilter("Go", newPos);
			}
		}
	}

	filterpositioning();
	
	showfilter(0, newPos);

	sendFocuserCorrection(EEData.m_FocusCorr[newPos]);
	
	sleepMotor();
} 

void manualChangeFilter(int8_t dir)
{
	if( filterposition < 0 )
	{
		findHomePosition(dir);
		return;
	}

	unsigned char newFilterpos = (filterposition + EEData.m_CountFilters + dir) % EEData.m_CountFilters;
	gotoFilter(newFilterpos | 0x80);
}

/*
void countFilters(void)
{
	EEData.m_CountFilters = 1;			//   

	showfilter("Found", EEData.m_CountFilters-1);

	wakeupMotor();

	goNextSensorFilterOff(1, 0);  //    1-  

	if( EEData.m_HomePresentInSensorFilter )
	{
		goNextSensorFilterOnRet(1);		//  Home    
		goNextSensorFilterOff(1, 0);	// 
	}

	goNextSensorFilterOnRet(1);    //     1- 
	while( goNextSensorFilterOff(1, 1) == 0 )  //  ,       home - 
	{
		EEData.m_CountFilters++;
		showfilter("Found", EEData.m_CountFilters-1);
		
		wakeupMotor(); //           
		goNextSensorFilterOnRet(1);
	}

	delay_0dot1sec(10);
} 
*/

#endif // _FILTERWHEEL_


//=========================================================================

#define maxCommandLen 200

unsigned char command[maxCommandLen];


enum eCommands
{
  eLock		   = 0,
  eUnlock	   = 1,
  eHome		   = 2,
  eCount	   = 3,
  eFilter	   = 4,
  eGoto		   = 5,
  eReadName	   = 6,
  eWriteName   = 7,
  eReadEE	   = 8,
  eWriteEE	   = 9,
  eTemperature = 10,
  eFocusPos    = 11,
  eFocusGo     = 12,
  eFocusSet    = 13,
  eIsAutoComp  = 14,
  eAutoCompSet = 15,
  eFocusMove   = 16,
  eFocusDirection = 17,
  eFocusSubstep   = 18,
  eMicrostepsRead  = 19,
  eMicrostepsWrite = 20,
  eLast		      = 21
};

unsigned char cmdSizes[eLast];
unsigned char cmdChars[eLast];

void fillCommands(void)
{
  cmdSizes[eLock]       = sizeof("FWLOCK")-1;
  cmdChars[eLock]       = 'L';
  cmdSizes[eUnlock]     = sizeof("FWUNLOCK")-1;
  cmdChars[eUnlock]     = 'U';
  cmdSizes[eHome]       = sizeof("FWHOME")-1;
  cmdChars[eHome]       = 'H';
  cmdSizes[eCount]      = sizeof("FWCOUNT")-1;
  cmdChars[eCount]      = 'C';
  cmdSizes[eFilter]     = sizeof("FWFILTER")-1;
  cmdChars[eFilter]     = 'F';
  cmdSizes[eGoto]       = sizeof("FWGOTO")-1+1;
  cmdChars[eGoto]       = 'G';
  cmdSizes[eReadName]   = sizeof("FWREADNAME")-1+1;
  cmdChars[eReadName]   = 'R';
  cmdSizes[eWriteName]  = sizeof("FWWRITENAME")-1+9;
  cmdChars[eWriteName]  = 'W';
  cmdSizes[eReadEE]     = sizeof("FWreadEE")-1 + 2;
  cmdChars[eReadEE]     = 'r';
  cmdSizes[eWriteEE]    = sizeof("FWwriteEE")-1 + 2;
  cmdChars[eWriteEE]    = 'w';
  
  cmdSizes[eTemperature] = sizeof("FWtemperature")-1 + 1;    // read temperature '0'...'8'
  cmdChars[eTemperature] = 't';
  cmdSizes[eFocusPos]    = sizeof("FWfocus")-1;        // read focus position
  cmdChars[eFocusPos]    = 'f';
  cmdSizes[eFocusGo]     = sizeof("FWgoFocus")-1 + 2;  // go to absolute focus position
  cmdChars[eFocusGo]     = 'g';
  cmdSizes[eFocusSet]    = sizeof("FWsetFocus")-1 + 2; // sets absolute focus position
  cmdChars[eFocusSet]    = 's';
  cmdSizes[eAutoCompSet] = sizeof("FWautoCompSet")-1 + 1; // sets auto temp compensation
  cmdChars[eAutoCompSet] = 'a';
  cmdSizes[eIsAutoComp]  = sizeof("FWisAuto")-1; // gets auto temp compensation
  cmdChars[eIsAutoComp]  = 'i';
  cmdSizes[eFocusMove]   = sizeof("FWmoveFocus")-1+2; // sets relative focus position
  cmdChars[eFocusMove]   = 'm';
  cmdSizes[eFocusDirection]   = sizeof("FWdirFocus")-1; // gets focuser active direction
  cmdChars[eFocusDirection]   = 'd';
  cmdSizes[eFocusSubstep]   = sizeof("FWubStep")-1+1; // gets or sets focuser substep size
  cmdChars[eFocusSubstep]   = 'u';

  cmdSizes[eMicrostepsRead]   = sizeof("FWzm")-1; // gets focuser microsteps
  cmdChars[eMicrostepsRead]   = 'z';
  cmdSizes[eMicrostepsWrite]   = sizeof("FWym")-1+ F_SUBSTEPS * 2; // sets focuser microsteps
  cmdChars[eMicrostepsWrite]   = 'y';
}

//
// FWLOCK                      FWL
// FWUNLOCK                    FWU
// FWHOME                      FWH     -
// FWCOUNT                     FWC     -  - . 
// FWFILTER                    FWF     -   . 
// FWGOTO[number]              FWG     - 
// FWREADNAME[x]               FWR     -    . 
// FWWRITENAME[x][8 char]      FWW     -    .
// 
// FWreadEE[addr][count]       FWr     -    eeprom
// FWwriteEE[addr][count][...] FWw     -    eeprom
//

unsigned char commandLen = 0;

unsigned char	eCmd = eLast;
char			writeEEcmd = 0;
uint16_t		command_counter = 0;
char			commandReady;

void command_interpreter (unsigned char cmd)
{
// 	lcd_xy(0,0);
// 	lcd_write_string_n (command, 16);
//  	lcd_xy(9,0);
//  	lcd_write_uint(cmd, 3);

	updateScreen = 1;

	switch (cmd)
	{
#ifdef _FILTERWHEEL_
	case eLock: 
		{ 
			automode=1;
//			lcd_xy(0,0);
//			lcd_write_string ("  RS232 Control ");
			usart_transmit('!');
			break;
		}
	case eUnlock:  
		{ 
			automode=0;
//			showfilter (0, filterposition);
			usart_transmit('!');
			break;
		}
	case eHome: 
		{
			gotoFilter(0);
			usart_transmit('0');
			break;
		}
	case eCount: 
		{ 
			usart_transmit(EEData.m_CountFilters + '0');
			break;
		}
	case eReadName:  // FWREADNAME[x]
		{ 
			unsigned char n = command[commandLen-1] - '0';
			if( n >= EEData.m_CountFilters )
			{
				usart_transmit('?');
			}
			else
			{
				for (unsigned char i=0; i<8; i++) 
					usart_transmit( EEData.m_Names[n][i] );
			}
			break;
		}
	case eWriteName:  // FWWRITENAME[x][name]
		{ 
			unsigned char n = command[commandLen-9] - '0';
			if( n >= EEData.m_CountFilters )
			{
				usart_transmit('?');
			}
			else
			{
				for (unsigned char i=0; i<8; i++) 
					EEData.m_Names[n][i] = (char)command[commandLen - 8 + i];
				writeSettingsToEEPROM();

//				EEPROM_WRITE_STRING( command + commandLen - 8, 8, 16 + n*8);
//				readSettingsFromEEPROM();
// 				for (i=0; i<8; i++) 
// 					EEData.m_Names[n][i] = command[commandLen-10 + i];
			}
			usart_transmit('!');
			break;
		}
	case eFilter:  
		{ 
			usart_transmit(filterposition + '0');
			break;
		}
	case eGoto: // FWGOTO[X] 
		{
			unsigned char filterpositionNew = command[commandLen-1] - '0';
			if( filterpositionNew < EEData.m_CountFilters )
			{
				gotoFilter(filterpositionNew);
				usart_transmit(filterposition + '0');
			}
			else
			{
				usart_transmit('?');
			}
			break;
		}
#endif // _FILTERWHEEL_
	case eReadEE: // FWreadEEPROM[a][c]
		{
			uint16_t addr = command[commandLen-2];
			uint8_t count = command[commandLen-1];
			for (; count > 0; count--) 
			{
				usart_transmit( EEPROM_READ(addr) );
				addr++;
			}
			break;
		}
	case eWriteEE: // FWwriteEEPROM[a][c][..]
		{
			const unsigned char cmdSize = cmdSizes[eWriteEE];
			uint16_t addr = command[cmdSize-2];
			uint16_t count = command[cmdSize-1];

// 			lcd_xy(7,0);
// 			lcd_write_char_hex(addr);
// 			lcd_dat(' ');
// 			lcd_write_char_hex(count);

			EEPROM_WRITE_STRING((char*)(command + cmdSize), count, addr);
			readSettingsFromEEPROM();

#ifdef _FILTERWHEEL_
			if( filterposition >= 0 )
				sendFocuserCorrection(EEData.m_FocusCorr[filterposition]);
#endif // _FILTERWHEEL_

			usart_transmit('!');
			break;				   
		}
#ifdef _FOCUSER_
		case eFocusPos: // FWfocus 
		{
			usart_transmit( focuser_step_counter & 0xff);
			usart_transmit((focuser_step_counter >> 8) & 0xff);
			break;
		}
		case eFocusGo: // FWgoFocus[nn] 
		{
			int16_t newpos = ((int16_t)command[commandLen-1] << 8) | command[commandLen-2];
			goFocusAbsolute(newpos);
			usart_transmit('!');
			break;
		}
		case eFocusMove: 
		{
			int16_t shift = ((int16_t)command[commandLen-1] << 8) | command[commandLen-2];
			goFocusRelative(shift);
			usart_transmit('!');
			break;
		}
		case eFocusSet: // FWsetFocus[nn] 
		{
			int16_t newFocus = ((int16_t)command[commandLen-1] << 8) | command[commandLen-2];
			//      

			setFocus(newFocus);

			wakeupFMotor();
			sleepFMotor();
			usart_transmit('!');
			break;
		}
		case eTemperature: // FWtemperature[X] 
		{
			int8_t Tn = command[commandLen-1] - '0';

			int16_t T;
			if( (Tn < EEData.m_TermometersCount) && (avrTCounts[Tn] > 0) )
				T = avrT[Tn] / Tscale;
			else
				T = 0x8000;

			usart_transmit( T & 0xff );
			usart_transmit((T >> 8) & 0xff);
			break;
		}
		case eAutoCompSet: // FWautoCompSet[X] 
		{
			setT_Kcompensation(command[commandLen-1] - '0');
			usart_transmit('!');
			break;
		}
		case eIsAutoComp: // FWisAuto 
		{
			usart_transmit(curTauto + '0');
			break;
		}
		case eFocusDirection:
		{
			usart_transmit(currFDirection > 0 ? '+' : '-');
			break;
		}
#ifdef F_MICROSTEP
		case eMicrostepsRead:
		{
			usart_transmit(F_SUBSTEPS * 2);
			for( int8_t k = 0; k < F_SUBSTEPS * 2; ++k )
			{
				usart_transmit(microStepPhase[k]);
			}
			break;
		}
		case eMicrostepsWrite:
		{
			for( int8_t k = 0; k < F_SUBSTEPS * 2; ++k )
			{
				microStepPhase[k] = command[commandLen-F_SUBSTEPS*2+k];
			}
			writeSettingsToEEPROM();
			sleepFMotor();
			usart_transmit('!');
			break;
		}
#endif

#ifdef _FILTERWHEEL_
		case eFocusSubstep:
		{
			char st = command[commandLen-1];
			if( st > 0 && st <= F_SUBSTEPS )
				EEData.m_FMotorStep = st;

			usart_transmit(EEData.m_FMotorStep);
			usart_transmit(F_SUBSTEPS);
			break;
		}
#endif

// 		case eSensorType: //FWSENSORTYPE?
// 		{
// 			int8_t st = command[commandLen-1] - '0';
// 			if ( ( st == 0 ) || ( st == 1 ) ) {
// 				EEData.m_TermometersType = st;
// 				writeSettingsToEEPROM();
// 			}
// 			usart_transmit(EEData.m_TermometersType + '0');
// 			break;
// 		}
#endif // _FOCUSER_
	}

	
	for(command_counter = commandLen-1; command_counter > 0; command_counter--) 
		command[command_counter] = 0;
	
	eCmd = eLast;
	command_counter = 0;
	commandLen = 0;
	commandReady = 0;
}


//	
// FWLOCK                      FWL
// FWUNLOCK                    FWU
// FWHOME                      FWH     -
// FWCOUNT                     FWC     -  - . 
// FWFILTER                    FWF     -   . 
// FWGOTO[number]              FWG     - 
// FWREADNAME[x]               FWR     -    . 
// FWWRITENAME[x][8 char]      FWW     -    .
// 
// FWreadEEPROM[addr][count]       FWr     -    eeprom
// FWwriteEEPROM[addr][count][...] FWw     -    eeprom
// 

SIGNAL (SIG_UART_RECV) 
{ 
  // USART RX interrupt command

	unsigned char ch = UDR;
	
//	lcd_xy(command_counter, 0);
//	lcd_dat(ch);

	if( commandReady )
		return;

	command[command_counter] = ch;
	command_counter++;
	
	switch(command_counter)
	{
	case 1:
		{
			commandLen = 0;
			writeEEcmd = 0;
			if( ch != 'F' )
			{
				command_counter = 0;
			}
			return;
		}
	case 2:
		{
			commandLen = 0;
			writeEEcmd = 0;
			if( ch != 'W' )
			{
				command_counter = 0;
			}
			return;
		}
	case 3:
		{
			commandLen = 0; 
			unsigned char k = 0;
			for( k = 0; k < eLast; ++k )
			{
				if( ch == cmdChars[k] )
				{
					eCmd = k;
					commandLen = cmdSizes[k];
					if( k == eWriteEE )
						writeEEcmd = 1;
					return;
				}
			}
			
			if( commandLen == 0 )
				command_counter = 0;
			return;
		}
	}


	if( writeEEcmd && command_counter >= commandLen ) // FWw[a][c]
	{
 		if( command[commandLen-1] > (maxCommandLen - commandLen) )
 			command[commandLen-1] = (maxCommandLen - commandLen);
		writeEEcmd = 0;
		commandLen += command[commandLen-1];
		return;
	}
	
	if( command_counter >= commandLen )
	{
		commandReady = 1;

//		command_interpreter(eCmd);
	}
}

//==============================================================================	

uint16_t speedModifyStep(uint16_t speed, char keyPlus, char keyMinus)
{
	if(keyPlus) 
	{
		if( speed < 100 )
			speed++;
		else
			speed += 10;
		
		if( speed > 1000 ) 
			speed = 1000;
	}
	
	if(keyMinus) 
	{
		if( speed <= 100 )
		{
			if( speed > 10 )
				speed--;
		}
		else
		{
			speed -= 10;
		}
	}
	return speed;
}

char keySpeedProcess(void)
{
	lcd_init();
	lcd_xy(0,0);
	lcd_write_eepromStr((uint16_t)&ee_keySpeedCaption);

	//   
	uint16_t oldSpeed = EEData.m_MotorSpeed;
	uint16_t oldSpeed2 = EEData.m_FMotorSpeed;
	while(keySpeed)
	{ 
		if( commandReady )
			command_interpreter(eCmd);

		updateScreen = 1;
		readkeys();
	
#ifdef _FILTERWHEEL_
		EEData.m_MotorSpeed = speedModifyStep(EEData.m_MotorSpeed, keySpeed && keyFilterNext, keySpeed && keyFilterPrev);

		lcd_xy(1,1);
		lcd_write_string("W=");
		lcd_write_uint(EEData.m_MotorSpeed, 4);
#endif

#ifdef _FOCUSER_
		EEData.m_FMotorSpeed = speedModifyStep(EEData.m_FMotorSpeed, keySpeed && keyFocuserNext, keySpeed && keyFocuserPrev);
		lcd_xy(10,1);
		lcd_write_string("F=");
		lcd_write_uint(EEData.m_FMotorSpeed, 4);
#endif
		
		delay_0dot1sec(2);
	}//   
	if (oldSpeed != EEData.m_MotorSpeed || oldSpeed2 != EEData.m_FMotorSpeed) 
	{
		writeSettingsToEEPROM();
	}

#ifdef _FOCUSER_
	sleepFMotor();
#endif
#ifdef _FILTERWHEEL_
	sleepMotor();
#endif

	lcd_clean_fast();

	return 1;
}


#ifdef _FILTERWHEEL_
char keyCorrectionProcess(void)
{
	if( filterposition < 0 )
		return 0;

	uint8_t sleep = 1;

	lcd_init();
	lcd_xy(0,0);
	lcd_write_eepromStr((uint16_t)&ee_keyCorrectionCap);

#ifdef _FOCUSER_
	wakeupFMotor();
#endif

	int16_t oldCorrection = EEData.m_PosCorr[filterposition];
	int16_t oldFocuserCorrection = EEData.m_FocusCorr[filterposition];
	while (keyCorrection)
	{ 
		if( commandReady )
			command_interpreter(eCmd);

		updateScreen = 1;
		readkeys();
		if(keyCorrection && keyFilterNext) 
		{
			EEData.m_PosCorr[filterposition]++;
			delay_0dot1sec(1);
		}
		if(keyCorrection && keyFilterPrev) 
		{
			EEData.m_PosCorr[filterposition]--;
			delay_0dot1sec(1);
		}

		int16_t fCorr = EEData.m_FocusCorr[filterposition];
		
		if(keyFocuserNext) 
		{
			if( fCorr > 10000 )
			{
				fCorr = 0;
			}
			else
			{
				
				if( fCorr == -1 )
				{
					fCorr = 20000; //  
					sleep = 3;
				}
				else
				{
					fCorr++;
				}
			}
		}
		if(keyFocuserPrev) 
		{
			if( fCorr == 0 )
			{
				fCorr = 20000; //  
				sleep = 3;
			} 
			else 
			{
				if( fCorr > 10000 )
					fCorr = -1;
				else
					fCorr--;
			}
		}
		
		EEData.m_FocusCorr[filterposition] = fCorr;

		lcd_xy(0,1);
		lcd_write_string("  ");
		lcd_write_int(EEData.m_PosCorr[filterposition], 4);
		lcd_write_string("   ");
		if( fCorr < 10000 )
			lcd_write_int(fCorr, 4);
		else
			lcd_write_string("none");
		lcd_dat(' ');

		sendFocuserCorrection(fCorr);
		
		delay_0dot1sec(sleep);
		sleep = 1;

	}//  
	if( oldCorrection != EEData.m_PosCorr[filterposition] )
	{
		writeSettingsToEEPROM();
		gotoFilter(filterposition);
	}
	if( EEData.m_FocusCorr[filterposition] != oldFocuserCorrection )
	{
		writeSettingsToEEPROM();
//		sendFocuserCorrection(EEData.m_FocusCorr[filterposition]);
	}

#ifdef _FOCUSER_
	sleepFMotor();
#endif

	lcd_clean_fast();

	return updateScreen;
}
#endif


#ifdef _FOCUSER_
char keyFocuserCorrProcess(void)
{
	lcd_init();
	lcd_xy(0,0);
	lcd_write_eepromStr((uint16_t)&ee_keyFocuserCap);

	uint16_t oldfocuserBacklash = EEData.m_FBacklash;
	int16_t oldtemperatureCompensationVal = EEData.m_FTK;

	while (keyFocuserCorr)
	{ 
		if( commandReady )
			command_interpreter(eCmd);

		updateScreen = 1;
		readkeys();
		if(keyFocuserCorr && keyFilterNext) 
		{
			EEData.m_FBacklash++;
		}
		if(keyFocuserCorr && keyFilterPrev) 
		{
			if( EEData.m_FBacklash > 0)
				EEData.m_FBacklash--;
		}
		
		if(keyFocuserCorr && keyFocuserNext) 
		{
			EEData.m_FTK++;
		}
		if(keyFocuserCorr && keyFocuserPrev) 
		{
			EEData.m_FTK--;
		}
		
		lcd_xy(0,1);
		lcd_write_string(" B=");
		lcd_write_uint(EEData.m_FBacklash, 4);
		lcd_write_string(" K=");
		lcd_write_int(EEData.m_FTK, 4);
		lcd_dat(' ');
		
		delay_0dot1sec(2);
	}//  

	if( oldfocuserBacklash != EEData.m_FBacklash || oldtemperatureCompensationVal != EEData.m_FTK )
	{
		writeSettingsToEEPROM();
	}

	lcd_clean_fast();

	return updateScreen;
}

char submenuProcess(void)
{
	uint8_t sleep = 10;
	lcd_clean_fast();
	do 
	{
		if( commandReady )
			command_interpreter(eCmd);

		lcd_xy(0,0);
		lcd_write_string("Tr=");
		if( referenceTemperatureReady )
			showTemperature(referenceTemperature / Tscale, 0,0);
		else
			lcd_write_string(" none ");
		
#if (LCD_WIDTH > 16)
		lcd_write_string(" Shift=");
		lcd_write_int(currFocusTermShift, 3);
		lcd_xy(0,1);
		lcd_write_string("Avr=");
		lcd_write_uint(EEData.m_TemperatureAverage, 3);
		lcd_write_string("   Revers=");
		lcd_write_uint(EEData.m_AutoTempMinReverseSteps, 3);
#else
		lcd_write_string(" Sr");
		lcd_write_int(currFocusTermShift, 3);
		lcd_xy(0,1);
		lcd_write_string("Avr=");
		lcd_write_uint(EEData.m_TemperatureAverage, 3);
		lcd_write_string("  Rev=");
		lcd_write_uint(EEData.m_AutoTempMinReverseSteps, 3);
#endif
		
		delay_0dot1sec(sleep);
		sleep = 2;
		readkeys();
		
		if(!keyCommand && keyFilterNext) 
		{
			int16_t step = EEData.m_TemperatureAverage < 20 ? 1 : 10;
			EEData.m_TemperatureAverage += step;
		}
		
		if(!keyCommand && keyFilterPrev && EEData.m_TemperatureAverage) 
		{
			int16_t step = EEData.m_TemperatureAverage <= 20 ? 1 : 10;
			EEData.m_TemperatureAverage -= step;
		}
		
		if(!keyCommand && keyFocuserNext) 
			EEData.m_AutoTempMinReverseSteps++;
		
		if(!keyCommand && keyFocuserPrev && EEData.m_AutoTempMinReverseSteps) 
			EEData.m_AutoTempMinReverseSteps--;
		
#ifdef _FILTERWHEEL_
	} while( !(keyCommand) );
#else
	} while( keyCorrection );
#endif

	lcd_clean_fast();
	writeSettingsToEEPROM();

	return 1;
}
#endif		


char keyCommandProcess(void)
{
	lcd_init();
	lcd_xy(0,0);
	lcd_write_eepromStr((uint16_t)&ee_keyCommandCap); //" Go  Reset K-T  "

	uint8_t sleep = 2;
#ifdef _FOCUSER_
	int8_t newTempCompensation = curTauto;
#endif

	while (keyCommand)
	{
		if( commandReady )
			command_interpreter(eCmd);

		readkeys();
		updateScreen = 1;

#ifdef _FILTERWHEEL_
		if (keyCommand && keyFilterNext) 
		{
			gotoFilter(0);
		}

		lcd_xy(0,1);
		lcd_write_string("Home ");
#endif

#ifdef _FOCUSER_
		if(keyCommand && keyFocuserNext) 
		{
			newTempCompensation = (newTempCompensation + 2) % (EEData.m_TermometersCount + 1) - 1;
			sleep = 5;
		}
		
		if(keyCommand && keyFilterPrev) 
		{
			setFocus(0);
			wakeupFMotor();
			sleepFMotor();
		}

		lcd_xy(5,1);
		if( focuser_step_counter != 0 )
			//" Go  Reset K-T  ");
			lcd_write_string("Focus ");
		else
			lcd_write_string("Done  ");
		
		if(	newTempCompensation < 0 )
		{
			lcd_write_string("Off  ");
		}
		else
		{
			lcd_dat(' ');
			lcd_dat('0' + newTempCompensation);
			lcd_write_string("   ");
		}

		if(keyCommand && keyFocuserPrev) 
		{
			return submenuProcess();
		}
#endif

		delay_0dot1sec(sleep);
		sleep = 2;
	}//  

#ifdef _FOCUSER_
	if( newTempCompensation != curTauto )
	{
		setT_Kcompensation(newTempCompensation);
	}
#endif

	lcd_clean_fast();

	return updateScreen;
}
		   
#ifdef HAVE_SHUTDOWN

#define EXTINT_LEVEL_LOW	0x00
#define EXTINT_EDGE_ANY		0x01
#define EXTINT_EDGE_FALLING	0x02
#define EXTINT_EDGE_RISING	0x03

void	init_INT0(void)
{
	GICR |= _BV(INT0); 
	GIFR |= _BV(INT0);
	MCUCR &= ~((1<<ISC01) | (1<<ISC00));
	MCUCR |= EXTINT_EDGE_FALLING;

	DDRD |= _BV(PD3);
}

SIGNAL(SIG_INTERRUPT0)
{
	if( !canShutdown )
		return;

	asm("cli");
	PORTD &= ~_BV(PD3);

	eeprom_write_word((uint16_t*)&ee_focuser_step_counter, focuser_step_counter);
	eeprom_write_word((uint16_t*)&ee_focuser_subshift, focuser_step_subshift);
	eeprom_write_word((uint16_t*)&ee_currFDirection, currFDirection + (currFBacklash << 8));
	eeprom_write_word((uint16_t*)&ee_focuserFilterShift, lastFocuserFilterShift);
	

// 	EEPROM_write_int((uint16_t)&ee_focuser_step_counter, focuser_step_counter);
// 	EEPROM_write((uint16_t)&ee_currFDirection, currFDirection);

	PORTD |= _BV(PD3);
	   
	lcd_clean_fast();
	lcd_xy(4,0);
	lcd_write_eepromStr((uint16_t)&ee_Shutdown);
	delay_0dot1sec(3);

	canShutdown = 0;

	while( bit_is_clear(PIND, PD2) );

	updateScreen = 1;
	asm("sei"); // enable interrupts
}

#endif


//================ ====================
int main(void)		  
{
#if defined(_FOCUSER_) && defined(_FILTERWHEEL_)
	PORTA = 0xff;
	DDRA = 0;
#endif 

	PORTB = 0xff;
	PORTC = 0xff;
	PORTD = 0xff;

	DDRB = 0;
	DDRC = 0;
	DDRD = 0;   // OC2 out

	portKeyLCD_D |= _BV(RS) | _BV(EN) | _BV(keyPinFocuserPrev) | _BV(keyPinFocuserNext) | _BV(keyPinFilterPrev) | _BV(keyPinFilterNext);

#ifdef _FOCUSER_
	portFMotorD |= allStepFBits;

#ifdef	TWO_WIRE
	i2PortOut	|= I2CL | I2DA;
//	i2PortD		|= I2CL | I2DA;
#endif // TWO_WIRE

#endif // _FOCUSER_

#ifdef _FILTERWHEEL_
	portMotorD |= allStepBits;
	portPowerD |= _BV(motorPowerOut);
#endif

	usart_init ();
	  	 

#ifdef F_MICROSTEP
	DDRD |= 0x80+0x10+0x20;   // out - OC2 OC1a OC1b

// 	TCCR2 = 0b01101001;
// 	OCR2 = 64;

//	TCCR1A = 0b11110001;
	TCCR1A = 0b11100001;   // 2-  
	TCCR1B = 0b00001001;

	OCR1A = 0;
	OCR1B = 0;

//	microStepPhase[0] = 0;
// 	for( int k = 1; k < F_SUBSTEPS * 2; ++k )
// 	{
// 		float val = sqrt(sin((k * 1.5707963) / F_SUBSTEPS)) * 255;
// 		int16_t ival = (int16_t)(val + 0.5);
// 		if( ival <= 0)
// 			ival = 0;
// 		if( ival > 255 )
// 			ival = 255;
// 
// 		microStepPhase[k] = ival;
// 	}
#endif	  

#ifdef HAVE_SHUTDOWN
	init_INT0();
	canShutdown = 0;
#endif

	readSettingsFromEEPROM();
	fillCommands();

	automode=0;
	commandLen = 0;
	commandReady = 0;
	command_counter = 0;
	eCmd = eLast;
	updateScreen = 1; 

	lcd_init();
	lcd_xy(0,1);
	lcd_write_eepromStr((uint16_t)&ee_t0);
	delay_0dot1sec(10);
	lcd_clean_fast();
	lcd_xy(0,0);
	lcd_write_eepromStr((uint16_t)&ee_t1);
	lcd_xy(0,1);
	lcd_write_eepromStr((uint16_t)&ee_t2);
	delay_0dot1sec(10);
	lcd_clean_fast();
	


#ifdef _FOCUSER_
	referenceTemperatureReady = 0;
	uint16_t count = 0;
	int8_t cT = 0;
	
#ifdef HAVE_SHUTDOWN
	focuser_step_counter	= EEPROM_READ_INT((uint16_t)&ee_focuser_step_counter);
	focuser_step_subshift	= EEPROM_READ_INT((uint16_t)&ee_focuser_subshift);
	currFDirection			= EEPROM_READ((uint16_t)&ee_currFDirection);
	lastFocuserFilterShift	= EEPROM_READ((uint16_t)&ee_focuserFilterShift);
#else
	focuser_step_counter = 0;
	focuser_step_subshift = 0;
	currFDirection = 1;
	lastFocuserFilterShift = EEData.m_FocusCorr[0];
#endif

	currFBacklash = (currFDirection > 0) ? EEData.m_FBacklash : 0;

	wakeupFMotor();
	sleepFMotor();
#endif 


#ifdef ONE_WIRE
	EEData.m_TermometersCount = DS18X20_search_sensors();
#endif 

#ifdef VOLTAGE_METER
	adc_init();
#endif // VOLTAGE_METER

	
#ifdef _FILTERWHEEL_
	filterposition = -1;
	findHomePosition(1);

//	wakeupMotor();
//	sleepMotor();
#endif 
    
// 	findHomePosition(1);
// 	lcd_write_string (t0);
// 	test=1;
// 	lcd_xy(0,1);
// 	lcd_write_string ("  Filter Counts ");
// 	countFilters();
// 	findHomePosition(1);
		 
	lcd_clean_fast();

	asm("sei"); //enable interrupts



	while (1)
	{
		if( commandReady )
			command_interpreter(eCmd);

#ifdef VOLTAGE_METER
		adc_start();
#endif // VOLTAGE_METER

#ifdef _FOCUSER_
		count++;
		if( (count % 10) == 1 )
		{
			cT = (cT+1) % EEData.m_TermometersCount;
//			for( int8_t i = 0; i < EEData.m_TermometersCount; ++i)
			{
				if( (readTemperature(cT) || (count % 710) == 1) && cT < 4 )
				{
					uint16_t integr = EEData.m_TemperatureAverage;
					if( integr < 2 )
						integr = 2;
					char processTK = (curTauto == cT) && ((avrTCounts[cT] % integr) == (integr-1));

#if (LCD_WIDTH > 16)
					lcd_xy( (cT > 1) ? 14 : 8, (cT & 0x1) );
					showAvrTemperature(cT, processTK);
#else
#ifdef THREE_SENSORS
					if ( ( (avrTCounts[cT] % 12) < 6) && (cT > 1) ) continue;
					lcd_xy( 9, (cT > 1) ? 1 : (cT & 0x1) );
					lcd_dat( 0x96 + cT );
#else
					lcd_xy( 10, (cT & 0x1) );
					//   1-  ,     
					if( cT < 2 || (avrTCounts[cT] && !avrTCounts[cT-2]) ) 
#endif
						showAvrTemperature(cT, processTK);
#endif 

					if( processTK )
						processT_K(avrT[cT]);
				}
			}
		}
// 		else
// 		{
// 			delay_0dot1sec(1);
// 		}
#endif 

		if (automode==1)
			continue;

		readkeys(); 

		if (!keySpeed && !keyCorrection && !keyFocuserCorr && !keyCommand)
		{
#ifdef _FILTERWHEEL_
			if (keyFilterPrev) 
			{
				manualChangeFilter(-1);
				updateScreen=1;
			}	
			if (keyFilterNext) 
			{
				manualChangeFilter(1);
				updateScreen=1;
			}

	#ifndef _FOCUSER_
			if( keyFocuserPrev )
			{
				gotoFilter(0);
				updateScreen=1;
			}
	#endif 

#endif 

#ifdef _FOCUSER_

#ifdef _FILTERWHEEL_
			if( keyFocuserNext || keyFocuserPrev )
#else
			if( keyFocuserNext || keyFocuserPrev || keyFilterPrev || keyFilterNext )
#endif
			{
				currFSpeedUp = 1;
				uint16_t steps = 0;
				wakeupFMotor();
				while(keyFocuserNext || keyFocuserPrev || keyFilterPrev || keyFilterNext)
				{
					readkeys();
					if( steps < 2 || keyFilterPrev || keyFilterNext )
						currFSpeed = 5;

					if( keyFocuserNext || keyFilterNext )
						doStepFMotor(1);
					if( keyFocuserPrev || keyFilterPrev )
						doStepFMotor(-1);
					showfocus();
					steps++;
				}
				sleepFMotor();
//				showfocus();
				currFSpeedUp = 5;
			}
#endif 
		} 

		if( keySpeed )
			updateScreen |= keySpeedProcess();

				  
#ifdef _FILTERWHEEL_
		if( keyCorrection )
			updateScreen |= keyCorrectionProcess();
#else
		if( keyCorrection )
			updateScreen |= submenuProcess();
#endif 

#ifdef _FOCUSER_
		if( keyCommand )
			updateScreen |= keyCommandProcess();

		if( keyFocuserCorr )
			updateScreen |= keyFocuserCorrProcess();


		if( updateScreen )
			showfocus();
#endif 

#ifdef _FILTERWHEEL_
	#ifdef VOLTAGE_METER
		if (( count % 2) == 1 )
		if( adc_read() || (( count % 50 ) == 1 )) {
			if( ( count % 200 ) >= 100 ) {
				lcd_xy(1,0);
				lcd_write_uint(voltage / 100, 2);
				lcd_dat('.');
				lcd_write_uint(voltage % 100, 2);
				lcd_dat('V');
			}
			else showfilter(0, filterposition);
		}
	#else
		if( updateScreen )
			showfilter(0, filterposition);
	#endif 
#endif 

		updateScreen = 0;
	}

}
