#ifndef _STEP_MOTORS_H
#define _STEP_MOTORS_H

#define STEPS_COUNT 8 

void delayMotorStep(char steps, uint16_t speed)
{
	int32_t delMks = (int32_t)507370L * steps;	 /*500000L*/
	delMks /= speed;
	delMks -= 90;
	delay_loop_mks(delMks);

	//	float delayMs = (500.0 * EEData.m_MotorStep) / speed - 0.43;
	//	_delay_ms(delayMs);
}

#ifdef _FILTERWHEEL_

unsigned char allStepBits = _BV(PHASE_A) | _BV(PHASE_B) | _BV(PHASE_C) | _BV(PHASE_D);

const unsigned char stepBits[STEPS_COUNT] = 
{
	_BV(PHASE_A) | _BV(PHASE_B),
	_BV(PHASE_B),
	_BV(PHASE_B) | _BV(PHASE_C),
	_BV(PHASE_C),
	_BV(PHASE_C) | _BV(PHASE_D),
	_BV(PHASE_D),
	_BV(PHASE_D) | _BV(PHASE_A),
	_BV(PHASE_A),
};

uint16_t	 currSpeed = 1;
int8_t		 currDirection = 0;

void doStepMotor2(int8_t direction, uint16_t speed )  // speed -     
{
	if( direction > 0 ) 
		filters_step_counter += EEData.m_MotorStep;
	if( direction < 0 ) 
		filters_step_counter -= EEData.m_MotorStep;

	unsigned char bitsOn = stepBits[filters_step_counter & 0x7];
	portMotor |= bitsOn;
	portMotor &= ~(allStepBits ^ bitsOn);

	delayMotorStep(EEData.m_MotorStep, speed);
}

void doStepMotor(int8_t direction )
{
	if( currDirection != direction )
	{
		currDirection = direction;
		currSpeed = EEData.m_MotorSpeed / 8 + 1;
	}

	doStepMotor2(direction, currSpeed);

	//   (  40 )
	if( currSpeed < EEData.m_MotorSpeed )
		currSpeed += (1 + EEData.m_MotorSpeed / 40);

	if( currSpeed > EEData.m_MotorSpeed )
		currSpeed = EEData.m_MotorSpeed;
}

void sleepMotor(void)
{
	if( EEData.m_MotorSleep )
		portMotor &= ~(allStepBits);

	currSpeed = EEData.m_MotorSpeed / 8 + 1;

	switch(EEData.m_MotorPowerBitType)
	{
	case 0: 
	case 2: 
		portPowerOut &= ~_BV(motorPowerOut); break;
	case 1:
	case 3: 
		portPowerOut |= _BV(motorPowerOut); break;
	}
}

void wakeupMotor(void)
{
	unsigned char bitsOn = stepBits[filters_step_counter & 0x7];
	portMotor |= bitsOn;
	portMotor &= ~(allStepBits ^ bitsOn);

	switch(EEData.m_MotorPowerBitType)
	{
	case 1:
	case 2:
		portPowerOut |= _BV(motorPowerOut); break;
	case 0: 
	case 3: 
		portPowerOut &= ~_BV(motorPowerOut); break;
	}

	currSpeed = EEData.m_MotorSpeed / 8 + 1;

	delayMotorStep(EEData.m_MotorStep, currSpeed);
}

#endif // _FILTERWHEEL_

#ifdef _FOCUSER_


const unsigned char allStepFBits = _BV(F_PHASE_A) | _BV(F_PHASE_B) | _BV(F_PHASE_C) | _BV(F_PHASE_D);


uint16_t	  currFSpeed = 1;
int8_t		  currFDirection = 1;
uint16_t	  currFBacklash =  0;
uint16_t	  currFSpeedUp = 5;


#ifdef F_MICROSTEP

//uint8_t microStepPhase[F_SUBSTEPS * 2] = {0}; //, 50, 98, 142, 180, 212, 236, 250, 255, 250, 236, 212, 180, 142, 98, 50 };


// #define F_SUBSTEPS 16
// 
// uint8_t microStepPhase[F_SUBSTEPS * 2] = {0, 25, 50, 74, 98, 120, 142, 162, 180, 197, 212, 225, 236, 244, 250, 254, 
// 									  255, 254, 250, 244, 236, 225, 212, 197, 180, 162, 142, 120, 98, 74, 50, 25 };

// sqrt(sin(i/16*pi/2))*255
// #define F_SUBSTEPS 16
// 
uint8_t microStepPhase[F_SUBSTEPS * 2] = {0, 80, 113, 137, 158, 175, 190, 203, 214, 224, 233, 239, 245, 249, 253, 254, 
										255, 254, 253, 249, 245, 239, 233, 224, 214, 203, 190, 175, 158, 137, 113, 80 };


const unsigned char stepFBits[STEPS_COUNT] = 
{
		_BV(F_PHASE_C) | _BV(F_PHASE_D),
		_BV(F_PHASE_D) | _BV(F_PHASE_A),
		_BV(F_PHASE_A) | _BV(F_PHASE_B),
		_BV(F_PHASE_B) | _BV(F_PHASE_C),
};

//#define SET_F_BITS																	
void SET_F_BITS(void)
{
	uint16_t fpos = (focuser_step_counter + currFBacklash + focuser_step_subshift) * EEData.m_FMotorStep;

	uint8_t phaseA = fpos % (F_SUBSTEPS*2);
	uint8_t phaseB = (phaseA + F_SUBSTEPS) % (F_SUBSTEPS*2);
	char bitsOn = stepFBits[(fpos / F_SUBSTEPS) & 0x03];
	portFMotor |= bitsOn;
	portFMotor &= ~(allStepFBits ^ bitsOn);

	OCR1A = microStepPhase[phaseA];
 	OCR1B = 255 - microStepPhase[phaseB];
	/*lcd_xy(0,0); lcd_write_uint(microStepPhase[phaseA],3); lcd_dat(' ');lcd_write_uint(microStepPhase[phaseB],3);*/
} //---------------------------------------------------------------------------------

void sleepFMotor(void)
{
	uint16_t fpos = (focuser_step_counter + currFBacklash + focuser_step_subshift) * EEData.m_FMotorStep;
	
	uint8_t phaseA = fpos % (F_SUBSTEPS*2);
	uint8_t phaseB = (phaseA + F_SUBSTEPS) % (F_SUBSTEPS*2);

	uint16_t valA = microStepPhase[phaseA];
	uint16_t valB = microStepPhase[phaseB];

	uint8_t sleepVal = EEData.m_FMotorSleep - 1;  // 0 -> 255, 1->0

	if( microStepPhase[0] & 0x80 )
	{
		valA = 255 - ((255-valA) * sleepVal) / 255;
		valB = 255 - ((255-valB) * sleepVal) / 255;
	}
	else
	{
		valA = (valA * sleepVal) / 255;
		valB = (valB * sleepVal) / 255;
	}

	OCR1A = valA;
  	OCR1B = 255 - valB;
	
//	lcd_xy(0,1); lcd_write_uint(valA,3); lcd_dat(' ');lcd_write_uint(valB,3); 

#ifdef HAVE_SHUTDOWN
	canShutdown = 1;
#endif
	
	currFSpeed = 5;
}


#else // F_MICROSTEP

const unsigned char stepFBits[STEPS_COUNT] = 
{
	_BV(F_PHASE_A) | _BV(F_PHASE_B),
		_BV(F_PHASE_B),
		_BV(F_PHASE_B) | _BV(F_PHASE_C),
		_BV(F_PHASE_C),
		_BV(F_PHASE_C) | _BV(F_PHASE_D),
		_BV(F_PHASE_D),
		_BV(F_PHASE_D) | _BV(F_PHASE_A),
		_BV(F_PHASE_A),
};

//#define SET_F_BITS															
void SET_F_BITS(void)
{
	char bitsOn = stepFBits[((focuser_step_counter + currFBacklash + focuser_step_subshift) * EEData.m_FMotorStep) & 0x07];
	portFMotor |= bitsOn;
	portFMotor &= ~(allStepFBits ^ bitsOn);
}

void sleepFMotor(void)
{
	if( EEData.m_FMotorSleep )
		portFMotor &= ~(allStepFBits);
	
#ifdef HAVE_SHUTDOWN
	canShutdown = 1;
#endif

	currFSpeed = 5;
}

#endif // F_MICROSTEP



void doStepFMotor2(int8_t direction, uint16_t speed )  // speed -     
{
	if( direction > 0 && bit_is_set(i2PortIn,SDA) ) 
		focuser_step_counter++;

	if( direction < 0 && bit_is_set(i2PortIn,SCL) ) 
		focuser_step_counter--;
	
	SET_F_BITS();

	delayMotorStep(1, speed);
}

void doStepFMotor(int8_t direction )   // direction -1 or 1
{
	if( currFDirection != direction )
	{
		currFDirection = direction;
		
		currFSpeed = 10;
		
		if( currFDirection > 0 && bit_is_set(i2PortIn,SDA) )
		{
			while( currFBacklash < EEData.m_FBacklash )
			{
				currFBacklash++;
				doStepFMotor2(0, currFSpeed);
				if( currFSpeed < EEData.m_FMotorSpeed )
					currFSpeed += 1;
			}
		}
		if( currFDirection < 0 && bit_is_set(i2PortIn,SCL) )
		{
			while( currFBacklash > 0 )
			{
				currFBacklash--;
				doStepFMotor2(0, currFSpeed);
				if( currFSpeed < EEData.m_FMotorSpeed )
					currFSpeed += 1;
			}
		}
	}
	
	doStepFMotor2(currFDirection, currFSpeed);
	
	if( currFSpeed < EEData.m_FMotorSpeed )
		currFSpeed += currFSpeedUp;
	
	//	if( currFSpeed > EEData.m_FMotorSpeed )
	//		currFSpeed = EEData.m_FMotorSpeed;
}

void wakeupFMotor(void)
{
#ifdef HAVE_SHUTDOWN
	canShutdown = 0;
#endif

	SET_F_BITS();
	
	currFSpeed = 5;

	delayMotorStep(1, currFSpeed);
}


void goFocusRelative(int16_t dif)
{
	if( dif == 0 )
		return;
	wakeupFMotor();
	for( ; dif > 0; dif-- )
		doStepFMotor(1);
	for( ; dif < 0; dif++ )
		doStepFMotor(-1);
	sleepFMotor();
}

void goFocusAbsolute(int16_t newPos)
{
	goFocusRelative( newPos - focuser_step_counter );
}

void setFocus(int16_t newFocus)
{
	int16_t round = F_SUBSTEPS*4 / EEData.m_FMotorStep;
	focuser_step_subshift = (focuser_step_subshift + focuser_step_counter - newFocus) % round;
	focuser_step_counter = newFocus;
}

#endif // _FOCUSER_

#endif // _STEP_MOTORS_H
